/*
 * gsampler.c
 * Copyright (C) Lilian Monchalin 2011 <isaya07@users.sf.net>
 * 
 * gsampler is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * gsampler is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#ifdef ENABLE_NLS
#include <locale.h>
#endif

#include <glib/gi18n.h>

#include "gsampler.h"
#include "gsampler-utils.h"
#include "gsampler-window.h"
#include "gsampler-option.h"
#include "gsampler-assistant.h"

static void
gsampler_first_start (GtkWidget* window, gpointer data)
{
	GtkWidget* assist = NULL;
	assist = GTK_WIDGET (data);
	g_return_if_fail (window != NULL);
	g_return_if_fail (assist != NULL);

	gtk_widget_destroy (assist);
	
	gdk_threads_add_idle_full (G_PRIORITY_DEFAULT_IDLE, gsampler_main_window_start_client,
	             	 		   g_object_ref (window), g_object_unref);	
}

#if (GTK_MAJOR_VERSION == 3)

G_DEFINE_TYPE (Gsampler, gsampler, GTK_TYPE_APPLICATION);

static void 
gsampler_activate (GApplication *application) {
	GtkWidget *window;
	GsamplerOption *option = NULL;
	GList* windows = NULL;
	g_return_if_fail (application != NULL);
	
	windows = gtk_application_get_windows ((GtkApplication*) application);
	if (windows != NULL) {
		window = (GtkWidget*) g_list_nth_data (windows, 0);
		gtk_window_present ((GtkWindow*) window);
	} else {
		g_set_application_name (_("Gsampler"));
		window = (GtkWidget*) gsampler_main_window_get_instance ();
		gtk_window_set_application ((GtkWindow*) window, (GtkApplication*) application);
		g_signal_connect ((GtkWidget*) window, "destroy", 
		                  (GCallback) gtk_main_quit, NULL);
		gtk_widget_show ((GtkWidget*) window);
		option = gsampler_option_get_instance ();
		if (gsampler_option_is_first_start (option))
		{
			GsamplerAssistant *assist = gsampler_assistant_new ();
			gtk_widget_show_all ((GtkWidget *) assist);
			g_signal_connect_swapped (assist, "close",
	                          	  	  (GCallback) gsampler_first_start, window);
		} else {
			gdk_threads_add_idle_full (G_PRIORITY_DEFAULT_IDLE, gsampler_main_window_start_client,
			             	 			g_object_ref (window), g_object_unref);
		}
		g_object_unref (option);

	}
}

static void 
gsampler_init (Gsampler * self) 
{
	
}

static void 
gsampler_finalize (GObject* obj) 
{
	G_OBJECT_CLASS (gsampler_parent_class)->finalize (obj);
}

static void 
gsampler_class_init (GsamplerClass * klass) 
{
	gsampler_parent_class = g_type_class_peek_parent (klass);
	G_APPLICATION_CLASS (klass)->activate = gsampler_activate;
	G_OBJECT_CLASS (klass)->finalize = gsampler_finalize;
}

Gsampler* 
gsampler_new (void) 
{
	g_type_init ();
	return g_object_new (GSAMPLER_TYPE_APPLICATION, 
	                      "application-id", "org.gnome.Gsampler", 
	                      "flags", G_APPLICATION_HANDLES_OPEN, NULL);
}

#else

static gboolean
gsampler_gtk2_assistant_start (gpointer data)
{
	GsamplerAssistant *assist;
	GtkWidget *window = GTK_WIDGET (data);
	g_return_val_if_fail (window != NULL, FALSE);

  	if (!g_source_is_destroyed (g_main_current_source ()))
    {	
		assist = gsampler_assistant_new ();
		gtk_widget_show_all ((GtkWidget *) assist);
		g_signal_connect_swapped (assist, "close",
			                   	  (GCallback) gsampler_first_start, window);
	}
	return FALSE;
}

#endif

int main (int argc, char ** argv) 
{
	
#ifdef ENABLE_NLS
	setlocale (LC_ALL, "");
	bindtextdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
	textdomain (GETTEXT_PACKAGE);
#endif
	
	/* Initialize threads */
	g_thread_init(NULL);
	gdk_threads_init ();
  		
	
	/* Init gtk+ */
	gtk_init (&argc, &argv);
	g_type_init ();

	//g_debug (" gtk : %d.%d.%d", GTK_MAJOR_VERSION, GTK_MINOR_VERSION, GTK_MICRO_VERSION);
	
#if ((GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION < 90) || GTK_MAJOR_VERSION < 2)
	GtkWidget* window;
	GsamplerOption *option = NULL;
	window = (GtkWidget*) gsampler_main_window_get_instance ();
	gtk_widget_show (window);
	option = gsampler_option_get_instance ();
	if (gsampler_option_is_first_start (option))
	{
		gdk_threads_add_idle_full (G_PRIORITY_DEFAULT_IDLE, gsampler_gtk2_assistant_start,
		                 			g_object_ref (window), g_object_unref);
	} else 
	{
		gdk_threads_add_idle_full (G_PRIORITY_DEFAULT_IDLE, gsampler_main_window_start_client,
			         	 			g_object_ref (window), g_object_unref);
	}
	g_object_unref (option);

	gdk_threads_enter ();
	gtk_main ();
	gdk_threads_leave ();
	return 0;
#else
	gint result = 0;
	Gsampler* app = NULL;
	gdk_threads_enter ();
	app = gsampler_new ();
	result = g_application_run ((GApplication*) app, argc, argv);
	g_object_unref (app);
	gdk_threads_leave ();
	return result;
#endif
}